""" get new domains """
import base64
from datetime import timedelta, date
import logging
import numpy as np
import os
import requests
import tabula
from time import sleep

from whoare.zone_parsers.ar.who import WhoAr
logger = logging.getLogger(__name__)


class NewDomains:
    """ Lee el nuevo formato de PDF implementado por el Boletìn Oficial desde el 
        28/8/2019: https://nic.opendatacordoba.org/static/28-08-2019.pdf
    """
    def __init__(self, ua="Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36"):
        self.base_domain = 'https://www.boletinoficial.gob.ar'
        self.home = 'seccion/cuarta'
        self.session = requests.Session()
        self.ua = ua
        self.data_path = 'whoare/zone_parsers/ar/data'

    def get_from_date_range(self, 
                            from_date=date.today() - timedelta(days=3),
                            to_date=date.today(),
                            push_url=None,
                            api_key=None):
        """ download and process last ~year 
            If push_url and api_key exists we send push domains"""

        dia = from_date
        full_results = {}

        c = 0
        errors = 0
        while dia < to_date:
            dia += timedelta(days=1)
            if dia.weekday() in [6, 7]:
                # skip weekends
                continue
            
            logger.info(f'Downloading {dia}')
            try:
                results = self.get_from_date(date=dia.strftime('%d-%m-%Y'))
            except Exception as e:
                logger.error(f'Error on date {dia}: {e}')
                raise

            logger.error(f"===============\n{dia.strftime('%d-%m-%Y')} {c} {errors}")
        
            # grabar cada lista de errores en un archivo
            for code, lista in results['errors'].items():
                f = open(f'{code}.txt', 'w')
                for dom in lista:
                    f.write(f'{dom}\n')
                f.close()

            # eliminar duplicados
            for zona, lista in results['zonas'].items():
                if zona not in full_results:
                    full_results[zona] = []
                for dom in lista:
                    if dom not in full_results[zona]:
                        full_results[zona].append(dom)

        final = []
        for zona, lista in full_results.items():
            f = open(f'{zona}.txt', 'w')
            for dom in lista:
                f.write(f'{dom}\n')
                final.append(dom)
            f.close()

        return final

    def push_domain(self, domain, push_url, api_key):
        """ push domain to external server """

        headers = {'Authorization': f'Token {api_key}'}
        data = {'dominio': domain}
        logger.info(f'Push new domain {domain} to {push_url}')
        response = requests.post(url=push_url, data=data, headers=headers)
        jresponse = response.json()
        logger.info(f' - Response [{response.status_code}]: {jresponse}')
        return response
        
    def get_from_date(self, date):
        """ Download specific date PDF and process it.
            sample date: 08-10-2019 DD-MM-YYYY
            """
        # check if exists
        final_pdf = os.path.join(self.data_path, f'{date}.pdf')

        if not os.path.isfile(final_pdf):
            
            # go inside home
            self.session.headers.update({"User-Agent": self.ua})
            try:
                self.session.get(f'{self.base_domain}/{self.home}')
                # call via ajax the required date
                internal_ajax_url = f'{self.base_domain}/edicion/actualizar/{date}'
                self.session.get(internal_ajax_url)
                home2_response = self.session.get(f'{self.base_domain}/{self.home}')
            except:
                logger.error('Error GET from BOE')
                return {'zonas': {}, 'errors': {}}
                
            if f"var fechaSeleccionada = '{date}';" not in home2_response.text:
                logger.error('Expected JS not foud')
                return {'zonas': {}, 'errors': {}}

            ret = self.download_pdf(date=date, path=final_pdf)
            if ret is None:
                return {'zonas': {}, 'errors': {}}
        
        else:
            logger.info(f'Skip download. File already exists {date} {final_pdf}')
        
        results = self.read_pdf(pdf_path=final_pdf)
        return results
        
    def download_pdf(self, date, path):
        """ descargar el PDF "actual" (la pagina guarda en sesion o algo similar el dia elegido) """

        if os.path.isfile(path):
            logger.info(f'Skip download. File already exists {date} {path}')
            return

        response = self.session.post(f'{self.base_domain}/pdf/download_section', data={'nombreSeccion': 'cuarta'})
        try:
            data = response.json()
        except:
            logger.error(f'INVALID JSON for {date} {path}')
            return None
        b64 = data['pdfBase64']
        bin_pdf = base64.b64decode(b64)
        f = open(path, 'wb')
        f.write(bin_pdf)
        f.close()
        return True

    def read_pdf(self, pdf_path):
        # Read pdf into list of DataFrame
        df = tabula.read_pdf(pdf_path, pages='all', lattice=True)
        valid_zones = WhoAr.zones()

        results = {'zonas': {}, 'errors': {}}
        last_dominio = 'zzzzz'
        last_zona = ''
        zona = ''
        valid_zone = False
        file_name = pdf_path.split('/')[-1]
        last_error_code = file_name
        c = 0
        last_df = None
        last_any_df = None
        
        for dom in df:
            logger.debug(f'\n************\nDF\t{dom.index}| {dom.columns} | {dom.values}\n**************')
            c += 1
            if len(dom.values) > 0:
                for dominio in dom.values:
                    if type(dominio[0]) == float:
                        if len(dominio) == 1 or type(dominio[1]) == float:
                            logger.error(f'Bad NAN line! \n\t{dom.index}\n\t{dom.columns}\n\t{dom.values}')
                            dom_name = ''
                        else:
                            dom_name = dominio[1]
                    else:
                        dom_name = dominio[0]

                    if dom_name != '' and dom_name == dom_name.lower() and ' ' not in dom_name and str(dom_name) != 'nan' and zona != '':
                        url = f"{dom_name}.{zona}"
                        if last_dominio > url and last_dominio[0] != url[0]:
                            if last_zona != zona:
                                logger.info(f'Changed zone OK {zona} at {dom_name}.{zona}')
                                last_zona = zona
                                valid_zone = True
                            else:
                                ix = '' if last_df is None else last_df.index
                                cl = '' if last_df is None else last_df.columns
                                ix2 = '' if last_any_df is None else last_df.index
                                cl2 = '' if last_any_df is None else last_df.columns
                                v2 = '[]' if last_any_df is None else last_df.values
                                logger.error(f'NO CHNGED ZONE "{last_dominio} > {url}" AND "{last_zona} != {zona}"')
                                # logger.error(f'Changed ZONE expected! \n\t{ix}\n\t{cl}\n\t{ix2}\n\t{cl2}\n\t{v2}\n\t{dom.index}\n\t{dom.columns}')
                                # skip all not-sure domains
                                valid_zone = False
                                last_error_code = f'error-{c}-{zona}-{file_name}'
                        
                        if valid_zone:
                            if zona not in results['zonas']:
                                results['zonas'][zona] = []
                            results['zonas'][zona].append(url)
                            logger.info(f'DOMAIN FOUND {url}')
                        else:
                            if last_error_code not in results['errors']:
                                results['errors'][last_error_code] = []
                            results['errors'][last_error_code].append(url)
                            logger.info(f'BAD DOMAIN {url} \n\t"{last_zona} != {zona}"\n\t"{last_dominio} > {url}"')
                        
                        last_dominio = url
                    else:
                        logger.error(f'Bad line! \n\t{zona}\n\t{dom.index}\n\t{dom.columns}\n\t{dom.values}')
                        
            skip = ['Unnamed', 'Boletín Oficial', 'NOMBRE DEL DOMINIO', 'Transferencias', 'Altas']
            ok = True
            for s in skip:
                if s in dom.columns[0]:
                    ok = False
                    logger.info(f'SKIP ZONE {dom.columns[0]}')  # \n\t{dom.index}\n\t{dom.columns}\n\t{dom.values}')
            
            last_any_df = dom

            if ok:
                new_zone = dom.columns[0]
                if new_zone == 'com.a': new_zone = 'com.ar' # (?)
                if new_zone == 'net.a': new_zone = 'net.ar' # (?)
                if new_zone == 'org.a': new_zone = 'org.ar' # (?)
                if new_zone == 'gob.a': new_zone = 'gob.ar' # (?)
                if new_zone == 'tur.a': new_zone = 'gob.ar' # (?)
                if new_zone == 'a': new_zone = 'ar' # (?)
                
                if new_zone not in valid_zones:
                    logger.error(f'BAD ZONE {new_zone}')
                else:
                    zona = new_zone
                    logger.info(f'Zona FOUND: {zona}')  # ' at {dom.columns}\n\t{dom.index}')

                last_df = dom

        return results



"""
Iterate over priority domains and send data to server
"""
import argparse
from datetime import date, timedelta
import logging
from time import sleep
import requests
import sys
from whoare.zone_parsers.ar.news import NewDomains


logger = logging.getLogger(__name__)


class WhoAreNews:
    def __init__(self, push_url, token):
        self.push_url = push_url  # destination URL to share news
        self.token = token
        
        self.total_analizados = 0
        self.nuevos = 0
        self.ya_existian = 0
    
    def run(self, days_ago=3):
        nd = NewDomains()

        fromd = date.today() - timedelta(days=days_ago)
        dominios = nd.get_from_date_range(from_date=fromd)

        for dominio in dominios:
            logger.info(f' {self.nuevos}+{self.ya_existian}={self.total_analizados}/{len(dominios)} Before push {dominio}')
            response = nd.push_domain(dominio, self.push_url, self.token)    
            
            self.total_analizados += 1
            jresponse = response.json()
            if response.status_code == 400:
                self.ya_existian += 1
            elif response.status_code in [200, 201]:
                pdid = jresponse.get('id', 0)
                if pdid == 0:
                    self.ya_existian += 1
                else:
                    self.nuevos += 1
            
            logger.info(f' - Response {jresponse}')
        

def main():

    push_url = 'https://nic.opendatacordoba.org/api/v1/dominios/predominio/'
    
    parser = argparse.ArgumentParser(prog='whoare-news')
    parser.add_argument('--push_url', nargs='?', help='URL to post results to', type=str, default=push_url)
    parser.add_argument('--token', nargs='?', help='Token to use as Header Autorization', type=str, required=True)
    parser.add_argument('--days_ago', nargs='?', help='Days ago to read', type=int, default=3)
    parser.add_argument('--log_level', nargs='?', default='INFO', type=str)
    
    args = parser.parse_args()

    if args.log_level == 'INFO':
        log_level = logging.INFO
    elif args.log_level == 'DEBUG':
        log_level = logging.DEBUG
    
    logger.setLevel(log_level)
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(log_level)
    # formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    formatter = logging.Formatter('%(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    was = WhoAreNews(push_url=args.push_url, token=args.token)

    was.run(days_ago=args.days_ago)


import csv
from datetime import timedelta, date
import logging
import os
import re
import requests
from time import sleep

from whoare.zone_parsers.ar.who import WhoAr
logger = logging.getLogger(__name__)


class NewDomains:
    """ leer los CSVs que se publican en blockchain
        https://rdlist.nic.ar/rd_list/
    """
    def __init__(self, ua="Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36"):
        self.base_domain = 'https://rdlist.nic.ar/rd_list/download'
        self.ua = ua
        self.data_path = 'whoare/zone_parsers/ar/data'

    def get_from_date_range(self, 
                            from_date=date.today() - timedelta(days=3),
                            to_date=date.today(),
                            push_url=None,
                            api_key=None):
        """ download and process last ~year 
            If push_url and api_key exists we send push domains"""

        dia = from_date
        full_results = {}

        c = 0
        errors = 0
        while dia < to_date:
            dia += timedelta(days=1)
            
            logger.info(f'Downloading {dia}')
            try:
                results = self.get_from_date(dated=dia)
            except Exception as e:
                logger.error(f'Error on date {dia}: {e}')
                raise

            logger.info(f"===============\n{dia} {c} {errors}")
        
            # grabar cada lista de errores en un archivo
            for code, lista in results['errors'].items():
                f = open(f'{code}.txt', 'w')
                for dom in lista:
                    f.write(f'{dom}\n')
                f.close()

            # eliminar duplicados
            for zona, lista in results['zonas'].items():
                if zona not in full_results:
                    full_results[zona] = []
                for dom in lista:
                    if dom not in full_results[zona]:
                        full_results[zona].append(dom)

        final = []
        for zona, lista in full_results.items():
            f = open(f'{zona}.txt', 'w')
            for dom in lista:
                f.write(f'{dom}\n')
                final.append(dom)
            f.close()

        return final

    def push_domain(self, domain, push_url, api_key):
        """ push domain to external server """

        headers = {'Authorization': f'Token {api_key}'}
        data = {'dominio': domain}
        logger.info(f'Push new domain {domain} to {push_url}')
        response = requests.post(url=push_url, data=data, headers=headers)
        jresponse = response.json()
        logger.info(f' - Response [{response.status_code}]: {jresponse}')
        return response
        
    def date_to_csvid(self, dated):
        """ los CSVs estan enumerados. Cada fecha es un numero
            1022 = 2020-12-15 

            del 303 pasa al 316 (?) 
            del 203 pasa al 312 y despues sigue con 204 (?) 
            
            NO SIRVE """

        # base = date(2020, 12, 25)
        # daydiff = (dated - base).days
        # return 1022 + daydiff

        # usar el filtro por fecha
        filter_url = f'https://rdlist.nic.ar/rd_list/?date_filter={dated.strftime("%d-%m-%Y")}'
        response = requests.get(filter_url)

        rgx = re.search(r'/rd_list/download/(?P<csvid>[0-9]+)/csv/', response.text)
        if rgx is not None:
            return rgx.group('csvid')
        
        logger.error(f'CSVID not found for {dated}')
        return None

    def get_from_date(self, dated):
        """ Download specific date PDF and process it.
            sample date: 08-10-2019 DD-MM-YYYY
            """
        # check if exists
        final_csv = os.path.join(self.data_path, f'{dated.strftime("%Y-%m-%d")}-blockchain.csv')

        if not os.path.isfile(final_csv):
                
            ret = self.download_csv(dated=dated, path=final_csv)
            if ret is None:
                return {'zonas': {}, 'errors': {}}
        
        else:
            logger.info(f'Skip download. File already exists {dated} {final_csv}')
        
        results = self.read_csv(csv_path=final_csv)
        return results
        
    def download_csv(self, dated, path):
        """ descargar el PDF "actual" (la pagina guarda en sesion o algo similar el dia elegido) """

        if os.path.isfile(path):
            logger.info(f'Skip download. File already exists {dated} {path}')
            return True

        headers = {"User-Agent": self.ua}
        csvid = self.date_to_csvid(dated)
        if csvid is None:
            # el 29/3/2019 por ejemplo no existe (?)
            return None

        url = f'{self.base_domain}/{csvid}/csv'
        try:
            response = requests.get(url, headers=headers)
        except:
            logger.error('Error GET from blockchain CSV')
            return None

        if response.status_code >= 400 or response.content is None or response.text == '':
            logger.error(f'Error getting {url} from blockchain CSV {response.status_code}')
            return None
    
        f = open(path, 'wb')
        f.write(response.content)
        f.close()

        logger.info(f'SAVED {url} to {path}')
        return True

    def read_csv(self, csv_path):
        logger.info(f'Reading CSV {csv_path}')
        results = {'zonas': {}, 'errors': {}}

        if not os.path.isfile(csv_path):
            return results
                
        f = open(csv_path)
        reader = csv.DictReader(f)
        # tipo	dominio	zona	id_dominio	titular	numero_doc	tipo_doc	fecha_registro
        # tipo = A (alta) T (transferencia)

        for row in reader:

            zona = row['zona']
            dominio = row['dominio']  # no incluye la zona
            if zona not in results['zonas']:
                results['zonas'][zona] = []
            
            # revisar dominios con catacteres espciales
            idna_dominio = dominio.encode().decode('idna')
            if dominio != idna_dominio:
                logger.info(f'Corrigiendo dominio con catacteres especiales {dominio} != {idna_dominio}')
                dominio = idna_dominio

            url = f'{dominio}.{zona}'
            results['zonas'][zona].append(url)
            logger.info(f'DOMAIN FOUND {url}')
        
        return results
